<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="2.0"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:wse="http://www.w3.org/2009/02/ws-evt"
                xmlns:xs="http://www.w3.org/2001/XMLSchema"
                xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
  <xsl:output method="xml" indent="yes"/>

  <!-- The "soapVer" paramater selects the version of SOAP used in the
       generated Event Sink WSDL. Valid values are "11" (for SOAP 1.1)
       and "12" (for SOAP 1.2); "11" is the default value. This
       template does not verify that this paramater has been set to a
       valid value. -->
  <xsl:param name="soapVer" select="11"/>

  <xsl:template match="@*|node()">
    <xsl:copy>
      <xsl:apply-templates select="@*|node()"/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="wse:EventDescriptions">
    <xsl:element name="wsdl:definitions"
                 namespace="http://schemas.xmlsoap.org/wsdl/">

      <!-- Rule 2: copy the targetNamespace from EventDescriptions -->
      <xsl:attribute name="targetNamespace">
        <xsl:value-of select="@targetNamespace"/>
      </xsl:attribute>
      <xsl:namespace name="tns" select="@targetNamespace"/>

      <!-- define xmlns:soap according to our SOAP version -->
      <xsl:choose>
        <xsl:when test="$soapVer=11">
          <xsl:namespace name="soap" select="'http://schemas.xmlsoap.org/wsdl/soap/'"/>
        </xsl:when>
        <xsl:when test="$soapVer=12">
          <xsl:namespace name="soap" select="'http://schemas.xmlsoap.org/wsdl/soap12/'"/>
        </xsl:when>
      </xsl:choose>

      <!-- define WS-Addressing 1.0 - Metadata namespace for use with wsam:Action -->
      <xsl:namespace name="wsam" select="'http://www.w3.org/2007/05/addressing/metadata'"/>

      <!-- Copy all the namespace nodes from wse:EventDescriptions to
           the wsdl:definitions element. This results in some
           extraneous namespace declarations in the Event Sink WSDL
           but it is crucial for picking up the namespace of the GED
           in wsdl:types/xs:schema. -->
      <xsl:for-each select="namespace::*">
        <xsl:copy-of select="."/>
      </xsl:for-each>

      <!-- generate a suffix for our portType, binding, etc. names -->
      <xsl:variable name="sec"
                    as="xs:decimal"
                    select="seconds-from-dateTime(current-dateTime())"/>

      <!-- assign names for portType, binding, etc. -->
      <xsl:variable name="portTypeName" select="concat('AutoGenPortType', $sec)"/>
      <xsl:variable name="bindingName" select="concat('AutoGenBinding', $sec)"/>
      <xsl:variable name="serviceName" select="concat('AutoGenService', $sec)"/>
      <xsl:variable name="portName" select="concat('AutoGenPort', $sec)"/>
      <xsl:variable name="listenURIName" select="concat('http://www.wstf.org/NotificationListeners/', $sec)"/>

      <!-- transform wse:types to wsdl:types -->
      <xsl:apply-templates select="wse:types"/>

      <!-- transform wse:eventType to a wsdl:messages -->
      <xsl:apply-templates select="wse:eventType" mode="messages"/>

      <!-- create wsdl:portType -->
      <xsl:element name="wsdl:portType"
                   namespace="http://schemas.xmlsoap.org/wsdl/">
        <xsl:attribute name="name">
          <xsl:value-of select="$portTypeName"/>
        </xsl:attribute>
        <xsl:apply-templates select="wse:eventType" mode="interface"/>
      </xsl:element>

      <!-- create a SOAP binding of the appropriate version -->
      <xsl:element name="wsdl:binding"
                   namespace="http://schemas.xmlsoap.org/wsdl/">
        <xsl:attribute name="name">
          <xsl:value-of select="$bindingName"/>
        </xsl:attribute>
        <xsl:attribute name="type">
          <xsl:value-of select="concat('tns:', $portTypeName)"/>
        </xsl:attribute>

        <!-- Include correct NS version of binding extension. The
             auto-generated wsdl:binding MUST be a Doc/literal style
             SOAP binding -->
        <xsl:choose>
          <xsl:when test="$soapVer=11">
            <xsl:element name="soap:binding"
                         namespace="http://schemas.xmlsoap.org/wsdl/soap/">
              <xsl:attribute name="style">document</xsl:attribute>
              <xsl:attribute name="transport">http://schemas.xmlsoap.org/soap/http</xsl:attribute>
            </xsl:element>
          </xsl:when>
          <xsl:when test="$soapVer=12">
            <xsl:element name="soap:binding"
                         namespace="http://schemas.xmlsoap.org/wsdl/soap12/">
              <xsl:attribute name="style">document</xsl:attribute>
              <xsl:attribute name="transport">http://schemas.xmlsoap.org/soap/http</xsl:attribute>
            </xsl:element>
          </xsl:when>
        </xsl:choose>

        <!-- fill in per-eventType operations -->
        <xsl:apply-templates select="wse:eventType" mode="binding"/>

      </xsl:element>

      <!-- create wsdl:service with the correct SOAP address extension-->
      <xsl:element name="wsdl:service"
                   namespace="http://schemas.xmlsoap.org/wsdl/">
        <xsl:attribute name="name">
          <xsl:value-of select="$serviceName"/>
        </xsl:attribute>
        <xsl:element name="wsdl:port"
                     namespace="http://schemas.xmlsoap.org/wsdl/">
          <xsl:attribute name="name">
            <xsl:value-of select="$portName"/>
          </xsl:attribute>
          <xsl:attribute name="binding">
            <xsl:value-of select="concat('tns:', $bindingName)"/>
          </xsl:attribute>
          <xsl:choose>
            <xsl:when test="$soapVer=11">
              <xsl:element name="soap:address"
                           namespace="http://schemas.xmlsoap.org/wsdl/soap/">
                <xsl:attribute name="location">
                  <xsl:value-of select="$listenURIName"/>
                </xsl:attribute>
              </xsl:element>
            </xsl:when>
            <xsl:when test="$soapVer=12">
              <xsl:element name="soap:address"
                           namespace="http://schemas.xmlsoap.org/wsdl/soap12/">
                <xsl:attribute name="location">
                  <xsl:value-of select="$listenURIName"/>
                </xsl:attribute>
              </xsl:element>
            </xsl:when>
          </xsl:choose>
        </xsl:element>
      </xsl:element>
    </xsl:element>
  </xsl:template>

  <!-- Rule 3: copy wse:types to wsdl:types -->
  <xsl:template match="wse:types">
    <xsl:element name="wsdl:types"
                 namespace="http://schemas.xmlsoap.org/wsdl/">
      <xsl:apply-templates select="@*|node()"/>
    </xsl:element>
  </xsl:template>

  <!-- Rule 4: each eventType becomes a wsdl:message -->
  <xsl:template match="wse:eventType" mode="messages">
    <xsl:element name="wsdl:message"
                 namespace="http://schemas.xmlsoap.org/wsdl/">
      <!-- the value of /wsdl:message/@name is generated from the
           wse:eventType/@name with "Msg" appended -->
      <xsl:attribute name="name">
        <xsl:value-of select="concat(@name,'Msg')"/>
      </xsl:attribute>
      <!-- all wsdl:messages have a single wsdl:part element (named
           "event") with the @element attribute equal to the value of
           wse:eventType/@element attribute -->
      <xsl:element name="wsdl:part"
                   namespace="http://schemas.xmlsoap.org/wsdl/">
        <xsl:attribute name="name">event</xsl:attribute>
        <xsl:attribute name="element">
          <xsl:value-of select="@element"/>
        </xsl:attribute>
      </xsl:element>
    </xsl:element>
  </xsl:template>

  <!-- Rule 5: each eventType becomes an in-only wsdl:portType/wsdl:operation -->
  <xsl:template match="wse:eventType" mode="interface">
    <xsl:element name="wsdl:operation"
                 namespace="http://schemas.xmlsoap.org/wsdl/">
      <!-- the name of the wsdl:operation is generated from the
           wse:eventType/@name with "Op" appended -->
      <xsl:attribute name="name">
        <xsl:value-of select="concat(@name,'Op')"/>
      </xsl:attribute>
      <!-- the wsdl:operation has a wsdl:input which is the message
           defined in (4) and no wsdl:output or wsdl:fault elements -->
      <xsl:element name="wsdl:input"
                   namespace="http://schemas.xmlsoap.org/wsdl/">
        <xsl:attribute name="message">
          <xsl:value-of select="concat('tns:', @name, 'Msg')"/>
        </xsl:attribute>
        <!-- each wsdl:operation has a @wsam:Action attribute with a
             value equal to the value of the wse:eventType/@action
             attribute -->
        <xsl:attribute name="wsam:Action"
                       namespace="http://www.w3.org/2007/05/addressing/metadata">
          <xsl:value-of select="@action"/>
        </xsl:attribute>
      </xsl:element>
    </xsl:element>
  </xsl:template>

  <!-- Rule 6: each eventType maps to a wsdl:binding/wsdl:operation -->
  <xsl:template match="wse:eventType" mode="binding">
    <xsl:element name="wsdl:operation"
                 namespace="http://schemas.xmlsoap.org/wsdl/">
      <xsl:attribute name="name">
        <xsl:value-of select="concat(@name,'Op')"/>
      </xsl:attribute>
      <xsl:choose>
        <xsl:when test="$soapVer=11">
          <xsl:element name="soap:operation"
                       namespace="http://schemas.xmlsoap.org/wsdl/soap/">
            <!-- for SOAP 1.1 bindings the soap:Operation/@soapAction
                 attribute for each operation is set to the empty
                 string -->
            <xsl:attribute name="soapAction"/>
          </xsl:element>
          <xsl:element name="wsdl:input"
                       namespace="http://schemas.xmlsoap.org/wsdl/">
            <xsl:element name="soap:body"
                         namespace="http://schemas.xmlsoap.org/wsdl/soap/">
              <xsl:attribute name="use">literal</xsl:attribute>
            </xsl:element>
          </xsl:element>
        </xsl:when>
        <xsl:when test="$soapVer=12">
          <!-- for SOAP 1.2 bindings the soap:Operation/@soapAction
               attribute is omitted -->
          <xsl:element name="soap:operation"
                       namespace="http://schemas.xmlsoap.org/wsdl/soap12/"/>
          <xsl:element name="wsdl:input"
                       namespace="http://schemas.xmlsoap.org/wsdl/">
            <xsl:element name="soap:body"
                         namespace="http://schemas.xmlsoap.org/wsdl/soap12/">
              <xsl:attribute name="use">literal</xsl:attribute>
            </xsl:element>
          </xsl:element>
        </xsl:when>
      </xsl:choose>
    </xsl:element>
  </xsl:template>

  <!-- strip xsi:schemaLocation -->
  <xsl:template match="@xsi:schemaLocation"/>

  <!-- strip any processing instructions -->
  <xsl:template match="processing-instruction()"/>

</xsl:stylesheet>
